@@ -1,11 +1,7 @@
-from __future__ import annotations
-
 from django.db import models
 from django.test import TestCase
 
-from django_fsm import FSMField
-from django_fsm import can_proceed
-from django_fsm import transition
+from django_fsm import FSMField, transition, can_proceed
 
 
 class BaseAbstractModel(models.Model):
@@ -19,18 +15,6 @@ class BaseAbstractModel(models.Model):
         pass
 
 
-class AnotherFromAbstractModel(BaseAbstractModel):
-    """
-    This class exists to trigger a regression when multiple concrete classes
-    inherit from a shared abstract class (example: BaseAbstractModel).
-    Don't try to remove it.
-    """
-
-    @transition(field="state", source="published", target="sticked")
-    def stick(self):
-        pass
-
-
 class InheritedFromAbstractModel(BaseAbstractModel):
     @transition(field="state", source="published", target="sticked")
     def stick(self):
@@ -58,4 +42,6 @@ class TestinheritedModel(TestCase):
 
     def test_field_all_transitions_works(self):
         transitions = self.model.get_all_state_transitions()
-        self.assertEqual({("new", "published"), ("published", "sticked")}, {(data.source, data.target) for data in transitions})
+        self.assertEqual(
+            set([("new", "published"), ("published", "sticked")]), set((data.source, data.target) for data in transitions)
+        )
