@@ -8,7 +8,7 @@ sys.path.insert(0, PROJECT_ROOT)
 
 if __name__ == "__main__":
     if len(sys.argv) == 1:
-        sys.argv += ['test']
+        sys.argv += ["test"]
 
     os.environ.setdefault("DJANGO_SETTINGS_MODULE", "tests.settings")
     execute_from_command_line(sys.argv)
